<?php

namespace Drp\Logger\Models;

use Auth;
use Drp\Logger\Traits\ConfigHelper;
use Illuminate\Database\Eloquent\Model;

/**
 * Class Logger
 * @package Drp\Logger
 */
class Logger extends Model
{
    use ConfigHelper;

    /**
     * Database table of Logger.
     *
     * @var string
     */
    protected $table = 'logs';

    /**
     * Fields that are converted to Carbon instances.
     *
     * @var array
     */
    protected $dates = [
        'created_at',
        'updated_at',
        'deleted_at',
    ];

    /**
     * Fields that can be mass assigned.
     *
     * @var array
     */
    protected $fillable = [
        'data',
        'type',
        'user_id',
    ];

    /**
     * Creating a new log entry.
     *
     * @param $data
     * @param null $type
     * @param null $userId
     * @return mixed
     */
    public function log($data, $type = null, $userId = null)
    {
        return $this->create([
            'data' => $data,
            'type' => $type,
            'user_id' => $this->determineUserId($userId),
        ]);
    }

    /**
     * @return mixed
     */
    public function user()
    {
        return $this->belongsTo($this->get('user'));
    }

    /**
     * Scope by user_id.
     *
     * @param $query
     * @param $id
     * @return mixed
     */
    public function scopeUserId($query, $id)
    {
        if ($this->present('user')) {
            return $query->with('user')->where('user_id', $id);
        }

        return $query->where('user_id', $id);
    }

    /**
     * Scope by type.
     *
     * @param $query
     * @param $type
     * @return mixed
     */
    public function scopeType($query, $type)
    {
        if ($this->present('user')) {
            return $query->with('user')->where('type', $type);
        }

        return $query->where('type', $type);
    }

    /**
     * Json Encode the data attribute.
     *
     * @param $value
     */
    protected function setDataAttribute($value)
    {
        $this->attributes['data'] = json_encode($value);
    }

    /**
     * Json Decode the data attribute.
     *
     * @param $value
     * @return mixed
     */
    protected function getDataAttribute($value)
    {
        return json_decode($value);
    }

    /**
     * Determines the user_id.
     *
     * @param $userId
     * @return null
     */
    private static function determineUserId($userId)
    {
        if (!is_null($userId)) {
            return $userId;
        }

        if (Auth::check()) {
            return Auth::user()->id;
        }

        return null;
    }
}
